/* 
 * A JavaScript implementation of the RSA Data Security, Inc. MD5 Message 
 * Digest Algorithm, as defined in RFC 1321. 
 * Version 1.1 Copyright (C) Paul Johnston 1999 - 2002. 
 * Code also contributed by Greg Holt 
 * See http://pajhome.org.uk/site/legal.html for details. 
 */

/* 
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally 
 * to work around bugs in some JS interpreters. 
 */
function safe_add(x, y)
{
    var lsw = (x & 0xFFFF) + (y & 0xFFFF)
    var msw = (x >> 16) + (y >> 16) + (lsw >> 16)
    return (msw << 16) | (lsw & 0xFFFF)
}

/* 
 * Bitwise rotate a 32-bit number to the left.  
 */
function rol(num, cnt)
{
    return (num << cnt) | (num >>> (32 - cnt))
}

/* 
 * These functions implement the four basic operations the algorithm uses. 
 */
function cmn(q, a, b, x, s, t)
{
    return safe_add(rol(safe_add(safe_add(a, q), safe_add(x, t)), s), b)
}
function ff(a, b, c, d, x, s, t)
{
    return cmn((b & c) | ((~b) & d), a, b, x, s, t)
}
function gg(a, b, c, d, x, s, t)
{
    return cmn((b & d) | (c & (~d)), a, b, x, s, t)
}
function hh(a, b, c, d, x, s, t)
{
    return cmn(b ^ c ^ d, a, b, x, s, t)
}
function ii(a, b, c, d, x, s, t)
{
    return cmn(c ^ (b | (~d)), a, b, x, s, t)
}

/* 
 * Calculate the MD5 of an array of little-endian words, producing an array 
 * of little-endian words. 
 */
function coreMD5(x)
{
    var a =  1732584193
    var b = -271733879
    var c = -1732584194
    var d =  271733878

    for(var i = 0; i < x.length; i += 16)
    {
        var olda = a
        var oldb = b
        var oldc = c
        var oldd = d

        a = ff(a, b, c, d, x[i+ 0], 7 , -680876936)
        d = ff(d, a, b, c, x[i+ 1], 12, -389564586)
        c = ff(c, d, a, b, x[i+ 2], 17,  606105819)
        b = ff(b, c, d, a, x[i+ 3], 22, -1044525330)
        a = ff(a, b, c, d, x[i+ 4], 7 , -176418897)
        d = ff(d, a, b, c, x[i+ 5], 12,  1200080426)
        c = ff(c, d, a, b, x[i+ 6], 17, -1473231341)
        b = ff(b, c, d, a, x[i+ 7], 22, -45705983)
        a = ff(a, b, c, d, x[i+ 8], 7 ,  1770035416)
        d = ff(d, a, b, c, x[i+ 9], 12, -1958414417)
        c = ff(c, d, a, b, x[i+10], 17, -42063)
        b = ff(b, c, d, a, x[i+11], 22, -1990404162)
        a = ff(a, b, c, d, x[i+12], 7 ,  1804603682)
        d = ff(d, a, b, c, x[i+13], 12, -40341101)
        c = ff(c, d, a, b, x[i+14], 17, -1502002290)
        b = ff(b, c, d, a, x[i+15], 22,  1236535329)

        a = gg(a, b, c, d, x[i+ 1], 5 , -165796510)
        d = gg(d, a, b, c, x[i+ 6], 9 , -1069501632)
        c = gg(c, d, a, b, x[i+11], 14,  643717713)
        b = gg(b, c, d, a, x[i+ 0], 20, -373897302)
        a = gg(a, b, c, d, x[i+ 5], 5 , -701558691)
        d = gg(d, a, b, c, x[i+10], 9 ,  38016083)
        c = gg(c, d, a, b, x[i+15], 14, -660478335)
        b = gg(b, c, d, a, x[i+ 4], 20, -405537848)
        a = gg(a, b, c, d, x[i+ 9], 5 ,  568446438)
        d = gg(d, a, b, c, x[i+14], 9 , -1019803690)
        c = gg(c, d, a, b, x[i+ 3], 14, -187363961)
        b = gg(b, c, d, a, x[i+ 8], 20,  1163531501)
        a = gg(a, b, c, d, x[i+13], 5 , -1444681467)
        d = gg(d, a, b, c, x[i+ 2], 9 , -51403784)
        c = gg(c, d, a, b, x[i+ 7], 14,  1735328473)
        b = gg(b, c, d, a, x[i+12], 20, -1926607734)

        a = hh(a, b, c, d, x[i+ 5], 4 , -378558)
        d = hh(d, a, b, c, x[i+ 8], 11, -2022574463)
        c = hh(c, d, a, b, x[i+11], 16,  1839030562)
        b = hh(b, c, d, a, x[i+14], 23, -35309556)
        a = hh(a, b, c, d, x[i+ 1], 4 , -1530992060)
        d = hh(d, a, b, c, x[i+ 4], 11,  1272893353)
        c = hh(c, d, a, b, x[i+ 7], 16, -155497632)
        b = hh(b, c, d, a, x[i+10], 23, -1094730640)
        a = hh(a, b, c, d, x[i+13], 4 ,  681279174)
        d = hh(d, a, b, c, x[i+ 0], 11, -358537222)
        c = hh(c, d, a, b, x[i+ 3], 16, -722521979)
        b = hh(b, c, d, a, x[i+ 6], 23,  76029189)
        a = hh(a, b, c, d, x[i+ 9], 4 , -640364487)
        d = hh(d, a, b, c, x[i+12], 11, -421815835)
        c = hh(c, d, a, b, x[i+15], 16,  530742520)
        b = hh(b, c, d, a, x[i+ 2], 23, -995338651)

        a = ii(a, b, c, d, x[i+ 0], 6 , -198630844)
        d = ii(d, a, b, c, x[i+ 7], 10,  1126891415)
        c = ii(c, d, a, b, x[i+14], 15, -1416354905)
        b = ii(b, c, d, a, x[i+ 5], 21, -57434055)
        a = ii(a, b, c, d, x[i+12], 6 ,  1700485571)
        d = ii(d, a, b, c, x[i+ 3], 10, -1894986606)
        c = ii(c, d, a, b, x[i+10], 15, -1051523)
        b = ii(b, c, d, a, x[i+ 1], 21, -2054922799)
        a = ii(a, b, c, d, x[i+ 8], 6 ,  1873313359)
        d = ii(d, a, b, c, x[i+15], 10, -30611744)
        c = ii(c, d, a, b, x[i+ 6], 15, -1560198380)
        b = ii(b, c, d, a, x[i+13], 21,  1309151649)
        a = ii(a, b, c, d, x[i+ 4], 6 , -145523070)
        d = ii(d, a, b, c, x[i+11], 10, -1120210379)
        c = ii(c, d, a, b, x[i+ 2], 15,  718787259)
        b = ii(b, c, d, a, x[i+ 9], 21, -343485551)

        a = safe_add(a, olda)
        b = safe_add(b, oldb)
        c = safe_add(c, oldc)
        d = safe_add(d, oldd)
    }
    return [a, b, c, d]
}

/* 
 * Convert an array of little-endian words to a hex string. 
 */
function binl2hex(binarray)
{
    var hex_tab = "0123456789abcdef"
    var str = ""
    for(var i = 0; i < binarray.length * 4; i++)
    {
        str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) +
            hex_tab.charAt((binarray[i>>2] >> ((i%4)*8)) & 0xF)
    }
    return str
}

/* 
 * Convert an array of little-endian words to a base64 encoded string. 
 */
function binl2b64(binarray)
{
    var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    var str = ""
    for(var i = 0; i < binarray.length * 32; i += 6)
    {
        str += tab.charAt(((binarray[i>>5] << (i%32)) & 0x3F) |
            ((binarray[i>>5+1] >> (32-i%32)) & 0x3F))
    }
    return str
}

/* 
 * Convert an 8-bit character string to a sequence of 16-word blocks, stored 
 * as an array, and append appropriate padding for MD4/5 calculation. 
 * If any of the characters are >255, the high byte is silently ignored. 
 */
function str2binl(str)
{
    var nblk = ((str.length + 8) >> 6) + 1 // number of 16-word blocks  
    var blks = new Array(nblk * 16)
    for(var i = 0; i < nblk * 16; i++) blks[i] = 0
    for(var i = 0; i < str.length; i++)
        blks[i>>2] |= (str.charCodeAt(i) & 0xFF) << ((i%4) * 8)
    blks[i>>2] |= 0x80 << ((i%4) * 8)
    blks[nblk*16-2] = str.length * 8
    return blks
}

/* 
 * Convert a wide-character string to a sequence of 16-word blocks, stored as 
 * an array, and append appropriate padding for MD4/5 calculation. 
 */
function strw2binl(str)
{
    var nblk = ((str.length + 4) >> 5) + 1 // number of 16-word blocks  
    var blks = new Array(nblk * 16)
    for(var i = 0; i < nblk * 16; i++) blks[i] = 0
    for(var i = 0; i < str.length; i++)
        blks[i>>1] |= str.charCodeAt(i) << ((i%2) * 16)
    blks[i>>1] |= 0x80 << ((i%2) * 16)
    blks[nblk*16-2] = str.length * 16
    return blks
}



function sha1(e) {
	function t(e, t) {
		var n = e << t | e >>> 32 - t;
		return n
	}

	function n(e) {
		var t = "",
			n, r, i;
		for (n = 0; n <= 6; n += 2) r = e >>> n * 4 + 4 & 15, i = e >>> n * 4 & 15, t += r.toString(16) + i.toString(16);
		return t
	}

	function r(e) {
		var t = "",
			n, r;
		for (n = 7; n >= 0; n--) r = e >>> n * 4 & 15, t += r.toString(16);
		return t
	}

	function i(e) {
		e = e.replace(/\r\n/g, "\n");
		var t = "";
		for (var n = 0; n < e.length; n++) {
			var r = e.charCodeAt(n);
			r < 128 ? t += String.fromCharCode(r) : r > 127 && r < 2048 ? (t += String.fromCharCode(r >> 6 | 192), t += String.fromCharCode(r & 63 | 128)) : (t += String.fromCharCode(r >> 12 | 224), t += String.fromCharCode(r >> 6 & 63 | 128), t += String.fromCharCode(r & 63 | 128))
		}
		return t
	}
	var s, o, u, a = new Array(80),
		f = 1732584193,
		l = 4023233417,
		c = 2562383102,
		h = 271733878,
		p = 3285377520,
		d, v, m, g, y, b;
	e = i(e);
	var w = e.length,
		E = new Array;
	for (o = 0; o < w - 3; o += 4) u = e.charCodeAt(o) << 24 | e.charCodeAt(o + 1) << 16 | e.charCodeAt(o + 2) << 8 | e.charCodeAt(o + 3), E.push(u);
	switch (w % 4) {
		case 0:
			o = 2147483648;
			break;
		case 1:
			o = e.charCodeAt(w - 1) << 24 | 8388608;
			break;
		case 2:
			o = e.charCodeAt(w - 2) << 24 | e.charCodeAt(w - 1) << 16 | 32768;
			break;
		case 3:
			o = e.charCodeAt(w - 3) << 24 | e.charCodeAt(w - 2) << 16 | e.charCodeAt(w - 1) << 8 | 128
	}
	E.push(o);
	while (E.length % 16 != 14) E.push(0);
	E.push(w >>> 29), E.push(w << 3 & 4294967295);
	for (s = 0; s < E.length; s += 16) {
		for (o = 0; o < 16; o++) a[o] = E[s + o];
		for (o = 16; o <= 79; o++) a[o] = t(a[o - 3] ^ a[o - 8] ^ a[o - 14] ^ a[o - 16], 1);
		d = f, v = l, m = c, g = h, y = p;
		for (o = 0; o <= 19; o++) b = t(d, 5) + (v & m | ~v & g) + y + a[o] + 1518500249 & 4294967295, y = g, g = m, m = t(v, 30), v = d, d = b;
		for (o = 20; o <= 39; o++) b = t(d, 5) + (v ^ m ^ g) + y + a[o] + 1859775393 & 4294967295, y = g, g = m, m = t(v, 30), v = d, d = b;
		for (o = 40; o <= 59; o++) b = t(d, 5) + (v & m | v & g | m & g) + y + a[o] + 2400959708 & 4294967295, y = g, g = m, m = t(v, 30), v = d, d = b;
		for (o = 60; o <= 79; o++) b = t(d, 5) + (v ^ m ^ g) + y + a[o] + 3395469782 & 4294967295, y = g, g = m, m = t(v, 30), v = d, d = b;
		f = f + d & 4294967295, l = l + v & 4294967295, c = c + m & 4294967295, h = h + g & 4294967295, p = p + y & 4294967295
	}
	var b = r(f) + r(l) + r(c) + r(h) + r(p);
	return b.toLowerCase()
}

function xor(e, t) {
	var n = "";
	for (var r = 0, i = e.length, s = t.length; r < i; r++) {
		if (r >= s) break;
		n += String.fromCharCode(e.charCodeAt(r) ^ t.charCodeAt(r))
	}
	return n
}

function base64encode(e) {
	var t = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
		n, r, i, s, o, u;
	i = e.length, r = 0, n = "";
	while (r < i) {
		s = e.charCodeAt(r++) & 255;
		if (r == i) {
			n += t.charAt(s >> 2), n += t.charAt((s & 3) << 4), n += "==";
			break
		}
		o = e.charCodeAt(r++);
		if (r == i) {
			n += t.charAt(s >> 2), n += t.charAt((s & 3) << 4 | (o & 240) >> 4), n += t.charAt((o & 15) << 2), n += "=";
			break
		}
		u = e.charCodeAt(r++), n += t.charAt(s >> 2), n += t.charAt((s & 3) << 4 | (o & 240) >> 4), n += t.charAt((o & 15) << 2 | (u & 192) >> 6), n += t.charAt(u & 63)
	}
	return n
}

/* 
 * External interface 
 */
function hexMD5 (str) { return binl2hex(coreMD5( str2binl(str))) }
function hexMD5w(str) { return binl2hex(coreMD5(strw2binl(str))) }
function b64MD5 (str) { return binl2b64(coreMD5( str2binl(str))) }
function b64MD5w(str) { return binl2b64(coreMD5(strw2binl(str))) }
/* Backward compatibility */
function calcMD5(str) { return binl2hex(coreMD5( str2binl(str))) }
module.exports = {
  hexMD5,
  hexMD5w,
  b64MD5,
  b64MD5w,
  sha1,
  xor,
  base64encode
} 